Imports System.IO

'
' Converts the text-based SOSOS Errorlog file into a Microsoft Access database
' that can be used to research program errors or WMI provider issues.
'
Public Class fmMain

    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        tbTableName.Text = Now.ToString("yyyyMMdd")
    End Sub

    Private Sub bnLogfile_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bnLogfile.Click
        OpenFileDialog1.InitialDirectory = "c:\"
        OpenFileDialog1.Filter = "Text files (*.txt)|*.txt"
        OpenFileDialog1.CheckFileExists = True

        If OpenFileDialog1.ShowDialog() = Windows.Forms.DialogResult.OK Then
            tbLogfile.Text = OpenFileDialog1.FileName
        End If
    End Sub

    Private Sub bnDatabase_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bnDatabase.Click
        OpenFileDialog1.InitialDirectory = "c:\"
        OpenFileDialog1.Filter = "Microsoft Access files (*.mdb)|*.mdb"
        OpenFileDialog1.CheckFileExists = False

        If OpenFileDialog1.ShowDialog() = Windows.Forms.DialogResult.OK Then
            tbDatabase.Text = OpenFileDialog1.FileName
        End If
    End Sub

    Private Sub bnDoit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bnDoit.Click
        Dim dt As DataTable
        Dim db As New Database
        Dim dr As DataRow
        Dim DDL_String, buf, location, temp As String
        Dim num, row_buffer As Integer
        Dim sr As StreamReader

        ' some quick sanity checks
        If Not File.Exists(tbLogfile.Text) Then
            ' create it!
            Try
                db.CreateDatabase(tbDatabase.Text)
            Catch ex As Exception
                Me.Cursor = Cursors.Default
                MsgBox("Can't create the database" & vbCr & ex.Message, MsgBoxStyle.Critical, "Error")
                Exit Sub
            End Try
        End If
        If tbTableName.Text = "" Then
            MsgBox("Must provide a table name", MsgBoxStyle.Critical, "Error")
            Exit Sub
        End If
        row_buffer = My.Settings.RowBuffer

        ' what the data table looks like
        DDL_String = "(ID int identity primary key, " _
        & "PCName varchar(50), " _
        & "RemoteFrom varchar(50), " _
        & "UserName varchar(50), " _
        & "ErrDate datetime, " _
        & "Version varchar(50), " _
        & "TableName varchar(50), " _
        & "DetailLevel varchar(50), " _
        & "ErrException varchar(100), " _
        & "ErrMsg varchar(150), " _
        & "Location varchar(255))"

        ' Let's open the database now to check for errors
        Me.Cursor = Cursors.WaitCursor
        Status.Text = "Opening Database"
        Application.DoEvents()
        Try
            db.CreateTable(tbDatabase.Text, tbTableName.Text, DDL_String)
            dt = db.OpenTable(tbDatabase.Text, tbTableName.Text)
        Catch ex As Exception
            Me.Cursor = Cursors.Default
            MsgBox("Can't open the database" & vbCr & ex.Message, MsgBoxStyle.Critical, "Error")
            Exit Sub
        End Try

        ' Let's open the log file to check for errors
        Try
            sr = New StreamReader(tbLogfile.Text)
        Catch ex As Exception
            Me.Cursor = Cursors.Default
            MsgBox("Can't open the error log file" & vbCr & ex.Message, MsgBoxStyle.Critical, "Error")
            Exit Sub
        End Try

        ' Parse the contents of the file
        buf = sr.ReadLine()
        num = 0
        dr = Nothing
        location = ""
        While (Not buf Is Nothing)
            Select Case Microsoft.VisualBasic.Left(buf, 3)
                Case "PCN"
                    If num > 0 Then
                        dr("Location") = Microsoft.VisualBasic.Left(location, 255)
                        dt.Rows.Add(dr)

                        ' keep 'em intertained
                        If num Mod 100 = 0 Then
                            Status.Text = dr("PCName").ToString
                            Application.DoEvents()
                        End If

                        ' periodically save data to reduce memory footprint
                        If dt.Rows.Count > row_buffer Then
                            db.UpdateTable(dt)
                            dt.Clear()
                        End If
                    End If

                    num += 1
                    dr = dt.NewRow
                    dr("PCName") = Split(buf, " ")(1).Trim.ToUpper
                Case "Rem"
                    temp = Split(buf, ":")(1).Trim.ToUpper
                    If temp <> "" Then
                        dr("RemoteFrom") = temp
                    End If
                Case "Use"
                    dr("UserName") = Split(buf, ":")(1).Trim
                Case "Dat"
                    dr("ErrDate") = CDate(buf.Substring(6))
                Case "Ver"
                    dr("Version") = Split(buf, ":")(1).Trim
                Case "Tab"
                    dr("TableName") = Split(buf, ":")(1).Trim
                Case "Det"
                    dr("DetailLevel") = Split(buf, ":")(1).Trim
                Case "Exc"
                    dr("ErrException") = Microsoft.VisualBasic.Left(Split(buf, ":")(1).Trim, 100)
                Case "Err"
                    dr("ErrMsg") = Microsoft.VisualBasic.Left(buf.Substring(7), 150)
                Case "Loc"
                    location = buf.Substring(10).Trim
                Case "   "
                    location &= " " & buf.Trim
            End Select
            buf = sr.ReadLine()
        End While

        If num > 0 Then
            If location <> "" Then
                dr("Location") = Microsoft.VisualBasic.Left(location, 255)
            End If
            dt.Rows.Add(dr)
        End If

        ' save the "tail" data
        If dt.Rows.Count > 0 Then
            Status.Text = "Saving Data"
            Application.DoEvents()
            db.UpdateTable(dt)
        End If
        db.Close()

        Me.Cursor = Cursors.Default
        Status.Text = "Done"
        MsgBox("Added " & num & " records to the database", MsgBoxStyle.Information, "Done!")
        Status.Text = "Ready..."
    End Sub

    Private Sub bnLaunch_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bnLaunch.Click
        If [File].Exists(tbDatabase.Text) Then
            System.Diagnostics.Process.Start(tbDatabase.Text)
        End If
    End Sub

    Private Sub bnQuit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bnQuit.Click
        Me.Close()
    End Sub
End Class